@@ -35,6 +35,8 @@
 #include "keycache.h"
 #include "keycache_p.h"
 
+#include "kleopatra_debug.h"
+
 #include "Libkleo/Predicates"
 
 #include "smimevalidationpreferences.h"
@@ -105,7 +107,7 @@ class KeyCache::Private
     friend class ::Kleo::KeyCache;
     KeyCache *const q;
 public:
-    explicit Private(KeyCache *qq) : q(qq)
+    explicit Private(KeyCache *qq) : q(qq), m_initalized(false)
     {
         connect(&m_autoKeyListingTimer, SIGNAL(timeout()), q, SLOT(startKeyListing()));
         updateAutoKeyListingTimer();
@@ -114,6 +116,7 @@ public:
     template < template <template <typename U> class Op> class Comp>
     std::vector<Key>::const_iterator find(const std::vector<Key> &keys, const char *key) const
     {
+        ensureCachePopulated();
         const std::vector<Key>::const_iterator it =
             std::lower_bound(keys.begin(), keys.end(), key, Comp<std::less>());
         if (it == keys.end() || Comp<std::equal_to>()(*it, key)) {
@@ -126,6 +129,7 @@ public:
     template < template <template <typename U> class Op> class Comp>
     std::vector<Subkey>::const_iterator find(const std::vector<Subkey> &keys, const char *key) const
     {
+        ensureCachePopulated();
         const std::vector<Subkey>::const_iterator it =
             std::lower_bound(keys.begin(), keys.end(), key, Comp<std::less>());
         if (it == keys.end() || Comp<std::equal_to>()(*it, key)) {
@@ -144,6 +148,7 @@ public:
         std::vector< std::pair<std::string, Key> >::const_iterator >
         find_email(const char *email) const
     {
+        ensureCachePopulated();
         return std::equal_range(by.email.begin(), by.email.end(),
                                 email, ByEMail<std::less>());
     }
@@ -170,6 +175,7 @@ public:
         std::vector<Key>::const_iterator
         > find_subjects(const char *chain_id) const
     {
+        ensureCachePopulated();
         return std::equal_range(by.chainid.begin(), by.chainid.end(),
                                 chain_id, _detail::ByChainID<std::less>());
     }
@@ -189,6 +195,8 @@ public:
         }
     }
 
+    void ensureCachePopulated() const;
+
 private:
     QPointer<RefreshKeysJob> m_refreshJob;
     std::vector<shared_ptr<FileSystemWatcher> > m_fsWatchers;
@@ -199,6 +207,7 @@ private:
         std::vector< std::pair<std::string, Key> > email;
         std::vector<Subkey> subkeyid;
     } by;
+    bool m_initalized;
 };
 
 shared_ptr<const KeyCache> KeyCache::instance()
@@ -273,6 +282,7 @@ void KeyCache::Private::refreshJobDone(const KeyListResult &result)
 {
     Q_EMIT q->keyListingDone(result);
     q->enableFileSystemWatcher(true);
+    m_initalized = true;
 }
 
 const Key &KeyCache::findByFingerprint(const char *fpr) const
@@ -291,23 +301,6 @@ const Key &KeyCache::findByFingerprint(const std::string &fpr) const
     return findByFingerprint(fpr.c_str());
 }
 
-std::vector<Key> KeyCache::findByFingerprint(const std::vector<std::string> &fprs) const
-{
-    std::vector<std::string> sorted;
-    sorted.reserve(fprs.size());
-    std::remove_copy_if(fprs.begin(), fprs.end(), std::back_inserter(sorted),
-                        boost::bind(is_string_empty(), boost::bind(&std::string::c_str, _1)));
-
-    std::sort(sorted.begin(), sorted.end(), _detail::ByFingerprint<std::less>());
-
-    std::vector<Key> result;
-    kdtools::set_intersection(d->by.fpr.begin(), d->by.fpr.end(),
-                              sorted.begin(), sorted.end(),
-                              std::back_inserter(result),
-                              _detail::ByFingerprint<std::less>());
-    return result;
-}
-
 std::vector<Key> KeyCache::findByEMailAddress(const char *email) const
 {
     const std::pair <
@@ -378,6 +371,7 @@ std::vector<Key> KeyCache::findByKeyIDOrFingerprint(const std::vector<std::strin
 
     std::vector<Key> result;
     result.reserve(keyids.size());   // dups shouldn't happen
+    d->ensureCachePopulated();
 
     kdtools::set_intersection(d->by.fpr.begin(), d->by.fpr.end(),
                               keyids.begin(), keyids.end(),
@@ -411,6 +405,7 @@ std::vector<Subkey> KeyCache::findSubkeysByKeyID(const std::vector<std::string>
     std::sort(sorted.begin(), sorted.end(), _detail::ByKeyID<std::less>());
 
     std::vector<Subkey> result;
+    d->ensureCachePopulated();
     kdtools::set_intersection(d->by.subkeyid.begin(), d->by.subkeyid.end(),
                               sorted.begin(), sorted.end(),
                               std::back_inserter(result),
@@ -636,6 +631,8 @@ std::vector<Key> KeyCache::findIssuers(std::vector<Key>::const_iterator first, s
     std::vector<Key> result;
     result.reserve(lastUniqueChainID - chainIDs.begin());
 
+    d->ensureCachePopulated();
+
     kdtools::set_intersection(d->by.fpr.begin(), d->by.fpr.end(),
                               chainIDs.begin(), lastUniqueChainID,
                               std::back_inserter(result),
@@ -764,6 +761,7 @@ void KeyCache::remove(const std::vector<Key> &keys)
 
 const std::vector<GpgME::Key> &KeyCache::keys() const
 {
+    d->ensureCachePopulated();
     return d->by.fpr;
 }
 
@@ -1024,7 +1022,7 @@ void KeyCache::RefreshKeysJob::Private::doStart()
 
 void KeyCache::RefreshKeysJob::Private::updateKeyCache()
 {
-    std::vector<Key> cachedKeys = m_cache->keys();
+    std::vector<Key> cachedKeys = m_cache->initialized() ? m_cache->keys() : std::vector<Key>();
     std::sort(cachedKeys.begin(), cachedKeys.end(), _detail::ByFingerprint<std::less>());
     std::vector<Key> keysToRemove;
     std::set_difference(cachedKeys.begin(), cachedKeys.end(),
@@ -1059,6 +1057,23 @@ Error KeyCache::RefreshKeysJob::Private::startKeyListing(const char *backend)
     return error;
 }
 
+bool KeyCache::initialized() const
+{
+    return d->m_initalized;
+}
+
+void KeyCache::Private::ensureCachePopulated() const
+{
+    if (!m_initalized) {
+        QEventLoop loop;
+        loop.connect(q, &KeyCache::keyListingDone,
+                     &loop, &QEventLoop::quit);
+        qCDebug(KLEOPATRA_LOG) << "Waiting for keycache.";
+        loop.exec();
+        qCDebug(KLEOPATRA_LOG) << "Keycache available.";
+    }
+}
+
 #include "moc_keycache_p.cpp"
 #include "moc_keycache.cpp"
 
